-- Indicator profile initialization routine
-- Defines indicator profile properties and indicator parameters
-- TODO: Add minimal and maximal value of numeric parameters and default color of the streams
function Init()
    indicator:name("DSS Bressert");
    indicator:description("DSS Bressert");
    indicator:requiredSource(core.Bar);
    indicator:type(core.Oscillator);
    indicator.parameters:addGroup("Calculation");
    indicator.parameters:addInteger("Frame", "Stochastic Period", "Stochastic Period", 13);
    indicator.parameters:addInteger("EMAFrame", "Smooth Period", "Smooth Period", 8);
	indicator.parameters:addInteger("SignalFrame", "Signal Period", "Signal Period", 8);
	
	indicator.parameters:addGroup("Style");
    indicator.parameters:addColor("Stochastic_color", "Color of Stochastic", "Color of Stochastic", core.rgb(255, 0, 0));
	indicator.parameters:addInteger("width1", "Line width", "", 1, 1, 5);
    indicator.parameters:addInteger("style1", "Line style", "", core.LINE_SOLID);
    indicator.parameters:setFlag("style1", core.FLAG_LINE_STYLE);
	
    indicator.parameters:addColor("Signal_color", "Color of Signal", "Color of Signal", core.rgb(0, 255, 0));
	indicator.parameters:addInteger("width2", "Line width", "", 1, 1, 5);
    indicator.parameters:addInteger("style2", "Line style", "", core.LINE_SOLID);
    indicator.parameters:setFlag("style2", core.FLAG_LINE_STYLE);
	
	
	 indicator.parameters:addGroup("OB/OS Levels");	
    indicator.parameters:addDouble("overbought", "Overbought Level","", 20);
    indicator.parameters:addDouble("oversold","Oversold Level","", 80);
	    indicator.parameters:addColor("level_overboughtsold_color", "Line Color","", core.rgb(128, 128, 128));
    indicator.parameters:addInteger("level_overboughtsold_width","Line width","", 1, 1, 5);
    indicator.parameters:addInteger("level_overboughtsold_style", "Line Style","", core.LINE_SOLID);
    indicator.parameters:setFlag("level_overboughtsold_style", core.FLAG_LEVEL_STYLE);
end

-- Indicator instance initialization routine
-- Processes indicator parameters and creates output streams
-- TODO: Refine the first period calculation for each of the output streams.
-- TODO: Calculate all constants, create instances all subsequent indicators and load all required libraries
-- Parameters block
local Frame;
local EMAFrame;
local SignalFrame;

local first;
local source = nil;

-- Streams block
local DSS= nil;
local EMA = nil;

local HIGH=nil;
local LOW=nil;

local DELTA=nil;
local MIT=nil;
local SmoothCoefficient=nil;

local Buffer=nil;
local Signal=nil;
local OUT=nil;

-- Routine
function Prepare()
    Frame = instance.parameters.Frame;
    EMAFrame = instance.parameters.EMAFrame;
	SignalFrame = instance.parameters.SignalFrame;
    source = instance.source;
    first = source:first();
	
	Buffer=instance:addInternalStream(0, 0);
	OUT=instance:addInternalStream(0, 0);
	

    local name = profile:id() .. "(" .. source:name() .. ", " .. Frame .. ", " .. EMAFrame ..", ".. SignalFrame.. ")";
    instance:name(name);
    DSS = instance:addStream("DSS", core.Line, name .. ".DSS", "DSS", instance.parameters.Stochastic_color, first);
	DSS:setWidth(instance.parameters.width1);
    DSS:setStyle(instance.parameters.style1);
	DSS:addLevel(instance.parameters.oversold, instance.parameters.level_overboughtsold_style, instance.parameters.level_overboughtsold_width, instance.parameters.level_overboughtsold_color);
	DSS:addLevel(instance.parameters.overbought, instance.parameters.level_overboughtsold_style, instance.parameters.level_overboughtsold_width, instance.parameters.level_overboughtsold_color);    

	
	EMA = core.indicators:create("EMA", OUT, SignalFrame);
	
    Signal = instance:addStream("SIGNAL", core.Line, name .. ".Signal", "Signal", instance.parameters.Signal_color, first);
	Signal:setWidth(instance.parameters.width2);
    Signal:setStyle(instance.parameters.style2);
	
	SmoothCoefficient= 2.0 / (1.0 + EMAFrame);
	
end

-- Indicator calculation routine
-- TODO: Add your code for calculation output values
function Update(period,mode)
	
    if period > Frame and source:hasData(period) then
	
	
	
			HIGH = core.max(source.high, core.range(period-Frame, period));
			LOW = core.min(source.low, core.range(period-Frame, period));
			DELTA = source.close[period] - LOW;
			MIT = DELTA/(HIGH - LOW)*100.0;
			Buffer[period] = SmoothCoefficient * (MIT - Buffer[period-1]) + Buffer[period-1];
			
			if period > 2*Frame  then
			
					HIGH = core.max(Buffer , core.range(period-Frame, period));
					LOW = core.min(Buffer , core.range(period-Frame, period));
					DELTA= Buffer[period] -LOW; 
					MIT = DELTA/(HIGH - LOW)*100.0;
					OUT[period] = SmoothCoefficient * (MIT - OUT[period-1]) + OUT[period-1];
					
					if period > 3*Frame + EMAFrame then
					DSS[period]=OUT[period];
					end
					
							if period > 3*Frame + SignalFrame then 					
							EMA:update(mode);
							Signal[period] = EMA.DATA[period];
							end
			elseif period == 2*Frame then 
            			
			OUT[period] =0;		
			end
	elseif period == Frame then
	Buffer[period] =0;	  	
    end
end

